classdef BayesianNetwork
% Bayesian Network

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

properties
    
VarNames % Variable names
ClassNames % Variable class names
Labels % Variable level labels

% optional properties of the BN

Description % A string describing the network
UserData % Additional information to be associated with the network

end

properties(Dependent)
    
NumNodes % Number of nodes

end

properties(SetAccess = protected)

G % The DAG as a NxN sparse connectivity matrix

NumLevels % Node number of levels

Param % Node parameters

CpdfParamFun % Node conditional distribution function parameter - returning functions
CpdfRndFun % Node conditional distribution random number generator functions

end

methods

function bnet = BayesianNetwork(G, nLevels, Param, CpdfParamFun, CpdfRndFun, varargin)
%BAYESIANNETWORK Create a Bayesian Network
%
%   BNET = BAYESIANNETWORK(G, NUMLEVELS, PARAM, CPDFPARAMFUN, CPDFRNDFUN)
%   creates a Bayesian Network with DAG G, node number of levels NUMLEVELS,
%   node parameters PARAM, node conditional distribution function
%   parameter - returning functions CPDFPARAMFUN and node conditional
%   distribution random number generator functions CPDFRNDFUN. G is a NxN 
%   sparse connectivity matrix, where M is the number of nodes of the
%   network. PARAM is a cell array of length M whose each cell contains the
%   parameters of the corresponding node. CPDFPARAMFUN is a a cell array of
%   function handles with length M, whose each cell contains a handle to a
%   function that accepts the corresponding parameters in PARAM and values
%   of the corresponding node's parents and returns the parameters of the 
%   conditional distribution function of the node given these values of its
%   parents. CPDFPARAMFUN can also be a single function handle; in this
%   case it is replicated to an 1-by-M cell array. CPDFRNDFUN is a a cell 
%   array of length M, whose each cell contains either a function handle or
%   one of the distribution names acceptable by the Statistics Toolbox 
%   RANDOM function. A function in CPDFRNDFUN accepts the parameters 
%   returned by the corresponding function in CPDFPARAMFUN and returns a
%   random vector from the conditional distribution function of the 
%   corresponding node. A distribution name in CPDFRNDFUN is passed to 
%   RANDOM along with the parameters returned by the corresponding function
%   in CPDFPARAMFUN and returns a random vector from the conditional
%   distribution function of the corresponding node. CPDFRNDFUN can also be
%   a single function handle or a distribution name; in this case it is
%   replicated to an 1-by-M cell array of array handles or strings
%   respectively.
%
%   BNET = BAYESIANNETWORK(G, NUMLEVELS, PARAM, CPDFPARAMFUN, CPDFRNDFUN, ...
%       'Param1',VAL1,'Param2',VAL2,...) specifies additional parameter 
%   name/value pairs chosen from the following:
%
%      'VarNames'       A cell array of M strings without duplicates 
%                       specifying the variable names. Default is 
%                       {'Var1', ..., 'VarN'}.
%
%      'ClassNames'     A cell array of M strings specifying the classes of 
%                       the variables. Valid class names are 'numeric',
%                       'nominal' and 'ordinal'. A single class name can
%                       also be given to be used for all variables. Default
%                       is 'nominal'.
%
%      'Labels'         A cell array of M cell arrays of strings specifying
%                       the level labels of the variables. The level labels
%                       of a numeric variable is the empty cell array ({}),
%                       since numeric variables do not have levels. The
%                       order by which the labels of an ordinal variable 
%                       are given specifies the order of its levels. 
%                       Default LABELS is {'Value1', ... , 'ValueL'} for
%                       each categorical variable. 
%
%      'Description'    A string describing the network. Default is [].
%
%      'UserData'       Any variable containing additional information to
%                       be associated with the network.

bnet.G = G;
bnet.Param = Param;
bnet.NumLevels = nLevels;

bnet.CpdfParamFun = CpdfParamFun;
bnet.CpdfRndFun = CpdfRndFun;
        
% Process optional arguments and do error checking

okargs =   {'VarNames', 'ClassNames', 'Labels', 'Description' 'UserData'};
defaults = {[], [], [], [], []};
[   eid emsg ...
    bnet.VarNames bnet.ClassNames bnet.Labels ...
    bnet.Description bnet.UserData ...
] = org.mensxmachina.commons.getargs(okargs, defaults, varargin{:});

if ~isempty(eid)
    error(sprintf('org:mensxmachina:pgm:BayesianNetwork:%s', eid), emsg);
end

end       

% public getters / setters

function bnet = set.Description(bnet, Description)

assert( isempty(Description) || ischar(Description), 'org:mensxmachina:pgm:BayesianNetwork:BadDescription', 'DESCRIPTION must be a string.' );

bnet.Description = Description;

end

function bnet = set.ClassNames(bnet, ClassNames)
    
if isempty(ClassNames)

    ClassNames = cell(1, bnet.NumNodes);
    
    % find indices of the nodes without number of levels
    isNumericNode = isnan(bnet.NumLevels);
    numNumericNodes = sum(isNumericNode, 2);
    
    if numNumericNodes > 0
        % set the class name of the nodes without number of levels to numeric
        ClassNames(isNumericNode) = repmat({'numeric'}, 1, numNumericNodes);
    end
    
    numCategoricalNodes = bnet.NumNodes - numNumericNodes;
    
    if numCategoricalNodes > 0
        % set the class names of the rest nodes to nominal
        ClassNames(~isNumericNode) = repmat({'nominal'}, 1, numCategoricalNodes);
    end

else

    good_classnames = true;

    if iscellstr(ClassNames) % a cell array of strings is given

        if length(ClassNames) == bnet.NumNodes

            good_classnames = ...
                all( arrayfun( @(i) ...
                        (isnan(bnet.NumLevels(i)) && strcmp(ClassNames{i}, 'numeric')) || ...
                        (~isnan(bnet.NumLevels(i)) && ismember(ClassNames{i}, {'nominal', 'ordinal'})), ...
                    1:bnet.NumNodes));

        else
            good_classnames = false;
        end

    elseif ischar(ClassNames) % a single string is given

        % find indices of the nodes without number of levels
        isNumericNode = isnan(bnet.NumLevels);

        if (all(isNumericNode) && strcmp(ClassNames, 'numeric')) || ...
           (all(~isNumericNode) && ismember(ClassNames, {'nominal', 'ordinal'}))
            
            % replicate the class name for all variables
            ClassNames = repmat({ClassNames}, 1, bnet.NumNodes);
            
        else
            good_classnames = false;
        end

    else
        good_classnames = false;
    end

    assert( good_classnames, ...
        'org:mensxmachina:pgm:BayesianNetwork:BadClassNames', ...
        ['CLASSNAMES must be consistent with the NUMLEVELS supplied: ' ...
         'the class name of a numeric variable (corresponding to a ' ...
         'missing (NaN) number of levels) must be ''numeric'' and the ' ...
         'class name of a categorical variable must be either ' ...
         '''nominal'' or ''ordinal''.'] ...
        );

end

bnet.ClassNames = ClassNames;

end

function bnet = set.Labels(bnet, Labels)

if isempty(Labels)

    % produce labels 'Value1', 'Value2', ... , 'ValueN'
    Labels = arrayfun(@(i) {sprintf('Value%d', i)}, ...
        1:bnet.NumNodes, 'UniformOutput', false);
    
else

    good_labels = ...
        iscell(Labels) && ... % a cell array
        length(Labels) == bnet.NumNodes && ... % of valid size
        all( arrayfun(@(i) ...
                iscellstr(Labels{i}) && ... % a cell array of strings
                isequal(sort(Labels{i}), unique(Labels{i})), ... % no duplicates
                1:length(Labels)) );

    assert( good_labels, ...
        'org:mensxmachina:pgm:BayesianNetwork:BadLabels', ...
        ['LABELS must be a cell array of length equal to the ' ...
         'number of nodes of the network. Each cell must contain a ' ...
         'cell array of strings, without duplicates, containing the ' ...
         'level labels of the corresponding variable.'] ...
        );

end

bnet.Labels = Labels;

end

function bnet = set.VarNames(bnet, VarNames)

if isempty(VarNames)

    % produce variable names 'Var1', 'Var2', ... , 'VarN'
    VarNames = arrayfun(@(i) sprintf('Var%d', i), ...
        1:bnet.NumNodes, 'UniformOutput', false);
    
else

    if iscellstr(VarNames)
        if isempty(VarNames)
            good_varnames = true;
        else
            good_varnames = ...
                length(VarNames) == bnet.NumNodes && ... % valid length
                all(cellfun(@isvarname, VarNames)) && ... % valid varnames
                isequal(sort(VarNames), unique(VarNames)); % no duplicates
        end
    else
        good_varnames = false;
    end

    assert( good_varnames, ...
        'org:mensxmachina:pgm:BayesianNetwork:BadVarNames', ...
        ['VARNAMES must either an empty cell array or ' ...
         'a cell array of unique valid variable names, ' ...
         'of length equal to the number of nodes.'] ...
        );

end

bnet.VarNames = VarNames;

end

% dependent property getters

function NumNodes = get.NumNodes(bnet)
    NumNodes = size(bnet.G, 1);
end

% protected setters / getters

function bnet = set.G(bnet, G)

assert( ...
    issparse(G) && graphisdag(G), ...
    'org:mensxmachina:pgm:BayesianNetwork:BadG', ...
    'G must be a DAG represented by a sparse connectivity matrix.' );

bnet.G = G;

end

function bnet = set.NumLevels(bnet, nLevels)

assert( ...
    isvector(nLevels) && ...
    length(nLevels) == bnet.NumNodes && ...
    all(org.mensxmachina.commons.ispositiveinteger(nLevels) | isnan(nLevels)), ...
    'org:mensxmachina:pgm:BayesianNetwork:BadNumLevels', ...
    ['NUMLEVELS must be a vector that contains positive integers ' ...
     ' and/or NaN values and has length equal to the number of nodes.'] ...
    );

bnet.NumLevels = nLevels;

end

function bnet = set.Param(bnet, Param)
       
assert( iscell(Param) && length(Param) == bnet.NumNodes, ...
    'org:mensxmachina:pgm:BayesianNetwork:BadParam', ...
    ['PARAM must be a cell array of length equal to the number of ' ...
     'nodes in the network.'] ...
    );

bnet.Param = Param;

end

function bnet = set.CpdfParamFun(bnet, CpdfParamFun)

if iscell(CpdfParamFun) && all(cellfun(@(cpdfParamFun) isa(cpdfParamFun, 'function_handle'), CpdfParamFun))

    % a cell array of function handles is given;
    % it must be of the right length
    good_cpdfParamFun = length(CpdfParamFun) == bnet.NumNodes;

elseif isa(CpdfParamFun, 'function_handle')

    % a single function handle is given; replicate it to an array
    good_cpdfParamFun = true;
    CpdfParamFun = repmat({CpdfParamFun}, 1, bnet.NumNodes);
    
else
    
    % invalid value
    good_cpdfParamFun = false;
    
end
    
assert( good_cpdfParamFun, ...
    'org:mensxmachina:pgm:BayesianNetwork:BadCpdfParamFun', ...
    ['CPDFPARAMFUN must be a cell array of function handles with ' ...
     'length equal to the number of nodes of the network.'] ...
    );

bnet.CpdfParamFun = CpdfParamFun;

end

function bnet = set.CpdfRndFun(bnet, CpdfRndFun)

if iscell(CpdfRndFun)

    % a cell array is given; it must be of the right length and each cell 
    % must be either a function handle or a string
    good_cpdfrndFun = ...
        length(CpdfRndFun) == bnet.NumNodes &&  ...
        all( cellfun(@(CpdfRndFun) ...
                isa(CpdfRndFun, 'function_handle') || ...
                (ischar(CpdfRndFun) && org.mensxmachina.pgm.BayesianNetwork.isdistname(CpdfRndFun)), ...
                CpdfRndFun ) );

elseif isa(CpdfRndFun, 'function_handle') || (ischar(CpdfRndFun) && org.mensxmachina.pgm.BayesianNetwork.isdistname(CpdfRndFun))

    good_cpdfrndFun = true;
    CpdfRndFun = repmat({CpdfRndFun}, 1, bnet.NumNodes);
    
else
    
    good_cpdfrndFun = false;
    
end
    
assert( good_cpdfrndFun, ...
    'org:mensxmachina:pgm:BayesianNetwork:BadCpdfRndFun', ...
    ['CPDFRNDFUN must be a cell array of length M, where M is the ' ...
     'number of nodes, whose each cell contains either a function ' ...
     'handle or one of the distribution names acceptable by the ' ...
     'Statistics Toolbox RANDOM function, or a single function handle ' ...
     'or distribution name'] );

bnet.CpdfRndFun = CpdfRndFun;

end

% other methods

function skeleton = skeleton(bnet)
%SKELETON Get the skeleton of a Bayesian Network
skeleton = org.mensxmachina.commons.graph.skeleton(bnet.G);
end

function parents = parents(bnet, i)
%PARENTS Get the list of parents of a Bayesian Network node      
parents = org.mensxmachina.commons.graph.parents(bnet.G, i);
end

function view(bnet)
%VIEW Draw figure of the network graph. Requires Bioinformatics Toolbox (TM).
view(biograph(bnet.G, bnet.VarNames));  
end

function viewskeleton(bnet)
%VIEWSKELETON Draw figure of the network graph skeleton. Requires Bioinformatics Toolbox (TM).
view(biograph(tril(bnet.skeleton), bnet.VarNames, 'ShowArrows', 'off')); 
end

function y = random(bnet, n)
%RANDOM Random matrix of observations from the distribution of a Bayesian Network
%
% Y = RANDOM(BNET, N) returns a random matrix Y of N observations
% from the distribution of Bayesian Network BNET.

fprintf('\nSampling...\n');

% get a topological order of the nodes
order = graphtopoorder(bnet.G); 

y = zeros(n, bnet.NumNodes);

k = 0;

for j = order % for each node in this order
    
    k = k + 1;
    
    fprintf('\nCreating column #%d, ''%s''...\n', j, bnet.VarNames{j});
    
    % get values of the node
    % according to its Param given its parents

    % get the parents of the node
    j_parents = parents(bnet, j);
    
    % get the conditional pdf parameters of the node for each sample
    cpdfParam = bnet.CpdfParamFun{j}(bnet.Param{j}, y(:, j_parents));
    
    % get a vector of random numbers chosen from the conditional
    % probability distribution of the node for each sample

    if ischar(bnet.CpdfRndFun{j})
        y(:, j) = random(bnet.CpdfRndFun{j}, cpdfParam{:}, n, 1);
    else
        y(:, j) = bnet.CpdfRndFun{j}(cpdfParam{:}, n, 1);
    end
    
end

end

function a = randomds(bnet, n)
%RANDOMDS Random dataset from the distribution of a Bayesian Network
%
% R = RANDOM(BNET, N) returns a random dataset R of N observations
% from the distribution of Bayesian Network BNET.

y = bnet.random(n);       

% initialize a cell array of dataset columns
dsCols = cell(1, bnet.NumNodes);

for j=1:bnet.NumNodes % for each node

    if strcmp(bnet.ClassNames{j}, 'numeric')
        
        % construct an numeric column
        dsCols{j} = y(:, j);
        
    elseif strcmp(bnet.ClassNames{j}, 'nominal')
        
        % construct an nominal column
        dsCols{j} = ...
            nominal( y(:, j), bnet.Labels{j}, 1:bnet.NumLevels(j) );
        
    else
        
        % construct a ordinal column
        dsCols{j} = ...
            ordinal( y(:, j), bnet.Labels{j}, 1:bnet.NumLevels(j) );
        
    end

end

if isempty(bnet.VarNames)
    params = {};
else
    params = [ 'VarNames' {bnet.VarNames} ];
end

% create a dataset consisting of the created columns
a = dataset(dsCols{:}, params{:});

end

end

methods(Static)

% save / load

function obj = loadobj(s)

obj = org.mensxmachina.pgm.BayesianNetwork( ...
    s.G, s.NumLevels, s.Param, s.CpdfParamFun, s.CpdfRndFun, ...
    'VarNames', s.VarNames, ...
    'Labels', s.Labels, 'ClassNames', s.ClassNames, ...
    'Description', s.Description, 'UserData', s.UserData);

end

function s = saveobj(obj)

s.G = obj.G;
s.NumLevels = obj.NumLevels;
s.Param = obj.Param;
s.CpdfParamFun = obj.CpdfParamFun;
s.CpdfRndFun = obj.CpdfRndFun;   
s.VarNames = obj.VarNames;
s.Labels = obj.Labels;
s.ClassNames = obj.ClassNames;
s.Description = obj.Description;
s.UserData = obj.UserData;

end

end

methods(Static, Access=private)

function tf = isdistname(name)
%ISDISTNAME Valid distribution names in cell array of strings 

distNames = {'beta', 'binomial', 'chi-square', 'extreme value', ...
             'exponential', 'f', 'gamma', 'generalized extreme value', ...
             'generalized pareto', 'geometric', 'hypergeometric', ...
             'lognormal', 'negative binomial', 'noncentral f', ...
             'noncentral t', 'noncentral chi-square', 'normal', 'poisson', ...
             'rayleigh', 't', 'discrete uniform', 'uniform', 'weibull'};

i = strmatch(lower(name), distNames);
if numel(i) > 1
    tf = false;
    return;
elseif numel(i) == 1
    name = distNames{i};
else % it may be an abbreviation that doesn't partially match the name
    name = lower(name);
end

abbrv = {'beta', 'binomial', 'chi2', 'chi-square', 'chisquare', 'ev', ...
         'extreme value', 'exponential', 'f', 'gamma', 'gev', ...
         'generalized extreme value', 'gp', 'generalized pareto', ...
         'geometric', 'hyge', 'hypergeometric', 'lognormal', 'nbin', ...
         'negative binomial', 'ncf', 'noncentral f', 'nct', ...
         'noncentral t', 'ncx2', 'noncentral chi-square', 'normal', ...
         'poisson', 'rayleigh', 't', 'unid', 'discrete uniform', ...
         'uniform', 'wbl', 'weibull'};

tf = ismember(name, abbrv);

end
    
end

end