classdef CategoricalBayesianNetwork < org.mensxmachina.pgm.BayesianNetwork
% Categorical Bayesian Network

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

methods

function bnet = CategoricalBayesianNetwork(G, numLevels, Param, varargin)
%CATEGORICALBAYESIANNETWORK Create a Categorical Bayesian Network
%   A Categorical Bayesian Network is a Bayesian Network whose all nodes
%   are categorical.
%
%   BNET = CATEGORICALBAYESIANNETWORK(G, NUMLEVELS, PARAM)
%   creates a Categorical Bayesian Network with DAG G, node number of
%   levels NUMLEVELS and node parameters PARAM. G is a NxN sparse
%   connectivity matrix, where M is the number of nodes of the network.
%   PARAM is a cell array of length M whose each cell contains the
%   parameters of the corresponding node in the form of a real ND-array 
%   with number of dimensions equal to the number of parents of the node 
%   plus one or equal to 2 if the node is a root node. The sums along the
%   1st dimension are all 1.
%
%   BNET = CATEGORICALBAYESIANNETWORK(G, NUMLEVELS, PARAM, ...
%       'Param1', VAL1,'Param2', VAL2,...) specifies additional parameter 
%   name/value pairs of the BAYESIANNETWORK class.
%
%   Example:
%
%       See ORG.MENSXMACHINA.PGM.WEATHER function code.
%
%   See also
%       ORG.MENSXMACHINA.PGM.BAYESIANNETWORK,
%       ORG.MENSXMACHINA.PGM.BIFREAD, ORG.MENSXMACHINA.PGM.DSL2PGM

% set all conditional distribution function parameter - returning functions
% to CATEGORICALCPDFPARAM
CpdfParamFun = ...
    repmat({@categoricalcpdfparam}, 1, size(G, 1));

% set all node conditional distribution random number generator functions
% to ORG.MENSXMACHINA.COMMONS.STATS.CATEGORICALRND
CpdfRndFun = ...
    repmat({@org.mensxmachina.commons.stats.categoricalrnd}, 1, size(G, 1));

% call superclass constructor
bnet = bnet@org.mensxmachina.pgm.BayesianNetwork( ...
    G, numLevels, Param, CpdfParamFun, CpdfRndFun, varargin{:});


% debug
%{
for i=1:length(Param)
%{
( ...  % correct dimension sizes
    ( numel(parents(bnet, i)) == 0 && isequal(size(Param{i}), [numLevels(i) 1]) ) || ...
    ( numel(parents(bnet, i)) > 0 && isequal(size(Param{i}), [numLevels(i) numLevels(parents(bnet, i))]) ) ...
)
%}
    %abs(reshape(sum(Param{i}, 1), 1, [])-1)
all(abs(reshape(sum(Param{i}, 1), 1, [])-1) <= numLevels(i)*eps(class(Param{i})))
end
%}

% deep check PARAM
goodParam = ...
    iscell(Param) && ...
    length(Param) == size(bnet.G, 1) && ...
    all( ...
        arrayfun(@(i) ...
            isreal(Param{i}) && ... % real array
            ( ...  % correct dimension sizes
                ( numel(parents(bnet, i)) == 0 && isequal(size(Param{i}), [numLevels(i) 1]) ) || ...
                ( numel(parents(bnet, i)) > 0 && isequal(size(Param{i}), [numLevels(i) numLevels(parents(bnet, i))]) ) ...
            ) && ... 
            ...%all(abs(reshape(sum(Param{i}, 1), 1, [])-1) < 1e-3), ... % probabilities sum to 1
            all(abs(reshape(sum(Param{i}, 1), 1, [])-1) <= numLevels(i)*eps(class(Param{i}))), ... % probabilities sum to 1
            1:length(Param)));
        
assert( goodParam, ...
    'org:mensxmachina:pgm:CategoricalBayesianNetwork:BadParam', ...
    ['PARAM must be a cell array of length equal to the number of ' ...
     'nodes in the network. PARAM{i} must be a real ND-array of size ' ...
     '[NUMLEVELS(i) 1] if the node is a root node and ' ...
     '[NUMLEVELS(i) NUMLEVELS(PARENTS(BNET, i))] otherwise. All sums ' ...
     'along the 1st dimension of PARAM{i} must be 1.'] ...
    );

end

end


methods(Static)

% save / load

function s = saveobj(obj)

% call superclass method
s = saveobj@org.mensxmachina.pgm.BayesianNetwork(obj);    

% no need to save the function handles; they are not used in loading
s = rmfield(s, {'CpdfParamFun', 'CpdfRndFun'});

end

function obj = loadobj(s)

obj = org.mensxmachina.pgm.CategoricalBayesianNetwork( ...
    s.G, s.NumLevels, s.Param, ...
    'VarNames', s.VarNames, ...
    'Labels', s.Labels, 'ClassNames', s.ClassNames, ...
    'Description', s.Description, 'UserData', s.UserData);

end

end

end