function tf = isdsepopt(G, G_prime, parentsInG, childrenInG, c, b, a)
%ISDSEPOPT Optimized version of ISDSEP
%   TF = ISDSEPOPT(G, GPRIME, PARENTSING, CHILDRENING, C, B, A) is the
%   optimized version of TF = ISDSEP(G, C, B, A). GPRIME is G + G', i.e.
%   the graph with links in G going in both directions. PARENTSING is a
%   cell array of length M containing the vector of parents ("inlist") of
%   each node, as returned by ORG.MENSXMACHINA.COMMONS.GRAPH.PARENTS. 
%   CHILDRENING is a cell array of length M containing the vector of 
%   children ("outlist") of each node, as returned by 
%   ORG.MENSXMACHINA.COMMONS.GRAPH.CHILDREN. GPRIME, PARENTSING and
%   CHILDRENING can be computed once and given to subsequent calls of
%   ISDSEPOPT.
%
%   See also ORG.MENSXMACHINA.PGM.ISDSEP

%   References:
%       [1] Richard E. Neapolitan. Learning Bayesian Networks. Prentice
%       Hall, April 2003.

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

import org.mensxmachina.commons.graph.*;

% get number of nodes
m = size(G, 1);

% create graph with G' labels
G_prime_label = sparse([], [], [], m, m, nnz(G_prime));

% detect nodes in A
inA = false(1, m);
inA(a) = true;

% detect nodes that are in A or have a descendent in A
isOrHasDescendentInA = false(1, m);
isOrHasDescendentInA(a) = true;
isOrHasDescendentInA(cell2mat(arrayfun(@(j) ancestors(G, j)', a, 'UniformOutput', false))) = true;

if ~isempty(c)
    open = c;
end

% initially set all nodes as d-separated from B given A
tf = true(1, m);

% set all nodes in B as not d-separated from B given A
tf(b) = false;

if ~isempty(c)
    
    open = setdiff(open, b);
    
    if isempty(open)
        tf = tf(c);
        return;
    end
    
end

% set all nodes in A as not d-separated from B given A
tf(a) = false;

if ~isempty(c)
    
    open = setdiff(open, a);
    
    if isempty(open)
        tf = tf(c);
        return;
    end
    
end

for j=b % for each node J in B

    % find children of J in G'
    jChildrenInG_prime = children(G_prime, j); 

    % set the children as not d-separated from B given A
    tf(jChildrenInG_prime) = false;

    if ~isempty(c)

        open = setdiff(open, jChildrenInG_prime);

        if isempty(open)
            tf = tf(c);
            return;
        end

    end

    % label the corresponding edges with 1
    G_prime_label(j, jChildrenInG_prime) = 1;
    
end

i = 1;
found = true;

while found

    found = false;
    
    % find i-labelled edges
    % [iLabelledEdgeParent iLabelledEdgeChild] = find(G_prime_label, i);
    [iLabelledEdgeParent iLabelledEdgeChild] = find(G_prime_label == i); % bug fixed
    
    for k=1:length(iLabelledEdgeParent) % for each i-labelled edge
        
        % get the corresponding child
        j = iLabelledEdgeChild(k);
        
        % get the corresponding parent
        j_parent = iLabelledEdgeParent(k);
        
        % initialize children of J in G' that are ends of already labelled
        % edges
        j_legalPathChildren = [];
        
        if ismember(j_parent, parentsInG{j}) % J_PARENT is a parent of J in G

            if isOrHasDescendentInA(j)
                j_legalPathChildren = parentsInG{j}';
            end

            if ~inA(j)
                % j_legalPathChildren = childrenInG{j};
                j_legalPathChildren = [j_legalPathChildren childrenInG{j}]; % bug fixed
            end

        else % J_PARENT is not a parent of J in G; J is a parent of J_PARENT in G

            if ~inA(j)
                j_legalPathChildren = [parentsInG{j}' childrenInG{j}];
            end

        end
        
        % keep children of J that are ends of unlabelled edges
        j_legalPathChildren = j_legalPathChildren(~G_prime_label(j, j_legalPathChildren));
        
        if ~isempty(j_legalPathChildren)
            
            % set them as not d-separated from B given A
            tf(j_legalPathChildren) = false;

            if ~isempty(c)

                open = setdiff(open, j_legalPathChildren);

                if isempty(open)
                    tf = tf(c);
                    return;
                end

            end
            
            % label the corresponding edges with i+1
            G_prime_label(j, j_legalPathChildren) = i + 1;
            
            % set the found flag
            found = true;
            
        end
        
    end   
    
    % i + 1 is the new i
    i = i + 1;
    
end

if ~isempty(c)
    
    tf = tf(c);

end

end