function [fdr q pi0] = skeletonfdr(outputSkeleton, p, varargin)
%SKELETONFDR Estimate the False Discovery Rate (FDR) of skeleton identification
%   SKELETONSTEPUP is a wrapper for ORG.MENSXMACHINA.COMMONS.STATS.STEPUP
%   specialized for Probabilistic Graphical Model (PGM) output skeleton
%   identification. Skeleton identification is viewed as a multiple testing
%   procedure, the null hypotheses being the absence of undirected edges.
%
%   Y = SKELETONFDR(OUTPUTSKELETON, P) computes Storey's [2] positive False
%   Discovery Rate (pFDR) estimators (with Lambda = 0) Y corresponding to
%   undirected edges in OUTPUTSKELETON with p-values P. The pFDR of an
%   undirected edge is the expected proportion of false undirected edges
%   when keeping only the undirected edges with p-values less than or equal
%   to the p-value of that undirected edge and throwing the rest away.
%   OUTPUTSKELETON is an M-by-M sparse adjacency matrix that represents the
%   skeleton, i.e. the graph with undirected edges in place of directed
%   edges, of a PGM's graph. P is an M-by-M sparse matrix containing the
%   p-values of the null hypotheses of absence of the edges in
%   OUTPUTSKELETON. P contains the p-value of each edge and 0 for each
%   non-edge in OUTPUTSKELETON. Y is an M-by-M sparse matrix that contains
%   the estimated FDR of each edge and 0 for each non-edge in
%   OUTPUTSKELETON.
%
%   [Y, Q] = SKELETONFDR(OUTPUTSKELETON, P) also returns the corresponding
%   q-values. Q is a is an M-by-M sparse matrix that contains the estimated
%   FDR of each edge and 0 for each non-edge in OUTPUTSKELETON.
%
%   [Y, Q, PI0] = SKELETONFDR(OUTPUTSKELETON, P) also returns the estimate
%   PI0 of the proportion of true undirected edges among all possible
%   undirected edges.
%
%   [...] = SKELETONFDR(OUTPUTSKELETON, P, VAR1, VAR2, VAR3, 'Param1', VAL1, 'Param2', VAL2, ...) 
%   specifies additional parameter name/value pairs chosen from the
%   following:
%
%      'PFdr'       A logical scalar that specifies whether the positive
%                   False Discovery Rate (pFDR) is estimated. If false, the
%                   False Discovery Rate (FDR) is estimated instead.
%                   Default is true.
%
%      'Lambda'     A numeric real scalar in [0, 1] that specifies the 
%                   value of the Lambda parameter of the (p)FDR estimators.
%                   Default is 0.
%
%      'Bycm'       A logical scalar that specifies whether the
%                   conservative modification of Benjamini and Yekutieli
%                   [4] is applied. Default is false.
%
%      'Thresholds' A numeric real vector of p-value thresholds (p)FDR is
%                   estimated at. When specified, Y and Q are numeric real
%                   vector of the same size as P.
%
%   See also ORG.MENSXMACHINA.COMMONS.STATS.FDR,
%   ORG.MENSXMACHINA.PGM.SKELETONRLZDFDR

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

import org.mensxmachina.commons.*;
import org.mensxmachina.commons.stats.*;

parser = inputParser;  % Create an instance of the inputParser class
parser.FunctionName = 'ORG.MENSXMACHINA.PGM.SKELETONFDR';
parser.KeepUnmatched = true;

parser.addRequired('outputSkeleton', @(outputSkeleton) issparse(outputSkeleton) && issquarematrix(outputSkeleton));
parser.addRequired('p', @(p) issparse(p) && isnumeric(p) && isreal(p) && issquarematrix(p));

parser.addParamValue('Lwhm', false, @(lwhm) isscalar(lwhm) && islogical(lwhm) );
parser.addParamValue('RandomOutputSkeleton', [], @(randomOutputSkeleton) iscell(randomOutputSkeleton) && isvector(randomOutputSkeleton) && all(cellfun(@(iRandomOutputSkeleton) issparse(iRandomOutputSkeleton) && issquarematrix(iRandomOutputSkeleton), randomOutputSkeleton)));
parser.addParamValue('RandomP', [], @(randomP) iscell(randomP) && isvector(randomP) && all(cellfun(@(iRandomP) issparse(iRandomP) && isnumeric(iRandomP) && isreal(iRandomP) && issquarematrix(iRandomP), randomP)));
parser.addParamValue('RandomTargetSkeleton', [], @(randomTargetSkeleton) iscell(randomTargetSkeleton) && isvector(randomTargetSkeleton) && all(cellfun(@(iRandomTargetSkeleton) issparse(iRandomTargetSkeleton) && issquarematrix(iRandomTargetSkeleton), randomTargetSkeleton)));

parser.parse(outputSkeleton, p, varargin{:});

Param = parser.Results;

% check consistency

assert( isequal(size(p), size(outputSkeleton)), ...
        'org:menxmachina:pgm:skeletonfdr:PSizeNotOutputSkeletonSize', ...
        'P must be of the same size as OUTPUTSKELETON.');

% simulation mode when RandomP is given and is not empty
simMode = ~isempty(Param.RandomOutputSkeleton);
    
if simMode % simulation mode
    
    b = length(Param.RandomOutputSkeleton); % # of permutations
    
    assert( all(cellfun(@(iRandomOutputSkeleton) isequal(size(iRandomOutputSkeleton), size(outputSkeleton)), Param.RandomOutputSkeleton)), ...
            'org:menxmachina:pgm:skeletonfdr:iRandomOutputSkeletonSizeNotSkeletonSize', ...
            'Each element of RANDOMOUTPUTSKELETON must be of the same size as OUTPUTSKELETON.');  

    assert( isequal(b, length(Param.RandomP)), ...
            'org:menxmachina:pgm:skeletonfdr:RandomPLengthNotSkeleton0Length', ...
            'RANDOMP must be of the same length as RANDOMOUTPUTSKELETON.');

    assert( all(cellfun(@(iRandomP) isequal(size(iRandomP), size(outputSkeleton)), Param.RandomP)), ...
            'org:menxmachina:pgm:skeletonfdr:iRandomPSizeNotSkeletonSize', ...
            'Each element of RANDOMP must be of the same size as OUTPUTSKELETON.');

    assert( isequal(b, length(Param.RandomTargetSkeleton)), ...
            'org:menxmachina:pgm:skeletonfdr:RandomTargetSkeletonLengthNotSkeleton0Length', ...
            'RANDOMTARGETSKELETON must be of the same length as RANDOMOUTPUTSKELETON.');

    assert( all(cellfun(@(iRandomTargetSkeleton) isequal(size(iRandomTargetSkeleton), size(outputSkeleton)), Param.RandomTargetSkeleton)), ...
            'org:menxmachina:pgm:skeletonfdr:RandomTargetSkeletonSizeNotSkeletonSize', ...
            'Each element of RANDOMTARGETSKELETON must be of the same size as OUTPUTSKELETON.');
    
end
   
% convert outputSkeleton matrix to vector
skeleton_vector = org.mensxmachina.commons.graph.skeleton2vector(outputSkeleton);

% convert maximum p-value matrix to vector
p_vector = org.mensxmachina.commons.graph.skeleton2vector(p);

% set m: number of hypotheses
if Param.Lwhm && ~simMode
    % Perform Li and Wang heuristic modification (non-simulation only)
    m = nnz(skeleton_vector); % set to number of discovered edges
else
    m = length(skeleton_vector); % set to number of possible edges
end

if simMode % simulation mode
    
    % convert random output output skeleton matrices to vectors
    randomOutputSkeleton_vector = cellfun(@org.mensxmachina.commons.graph.skeleton2vector, Param.RandomOutputSkeleton, 'UniformOutput', false);
    
    % convert random p-value matrices to vectors
    randomP_vector = cellfun(@org.mensxmachina.commons.graph.skeleton2vector, Param.RandomP, 'UniformOutput', false);

    % convert random target output skeleton matrices to vectors
    randomTargetSkeleton_vector = cellfun(@org.mensxmachina.commons.graph.skeleton2vector, Param.RandomTargetSkeleton, 'UniformOutput', false); 
        
end

% get edge linear indices in output skeleton vector
skeleton_vector_ind = find(skeleton_vector);

% keep only maximum p-values corresponding to edges and convert to full
p_skeleton_vector = full(p_vector(skeleton_vector_ind));

if simMode % simulation mode

    % get edge linear indices in random output output skeleton vectors
    randomOutputSkeleton_vector_ind = cellfun(@find, randomOutputSkeleton_vector, 'UniformOutput', false);

    % get edge linear indices in random target output skeleton vectors
    randomTargetSkeleton_vector_ind = cellfun(@find, randomTargetSkeleton_vector, 'UniformOutput', false);
    
    % get their differrence,
    % the random false discovery output skeleton vector indices
    randomFDSkeleton_vector_ind = arrayfun(@(i) setdiff(randomOutputSkeleton_vector_ind{i}, randomTargetSkeleton_vector_ind{i}), 1:b, 'UniformOutput', false);
    
    % keep only false discovery p-values and convert to full
    randomP_fd_skeleton_vector = arrayfun(@(i) full(randomP_vector{i}(randomFDSkeleton_vector_ind{i})), 1:b, 'UniformOutput', false);
    
    % get # of null p-values for each random outputSkeleton
    m0 = cellfun(@(iRandomTargetSkeleton) m - nnz(iRandomTargetSkeleton)/2, Param.RandomTargetSkeleton);
    
else    
    randomP_fd_skeleton_vector = [];  
    m0 = [];
end

% create varargin for FDR
Unmatched_fieldnames = fieldnames(parser.Unmatched)';
numUnmatched = length(Unmatched_fieldnames);

varargin_fdr = cell(1, 2*numUnmatched);
varargin_fdr(1:2:2*numUnmatched) = Unmatched_fieldnames;
varargin_fdr(2:2:2*numUnmatched) = struct2cell(parser.Unmatched);

% compute FDR corresponding to these maximum p-values only
if nargout == 1
    fdr_skeleton_vector = org.mensxmachina.commons.stats.fdr(p_skeleton_vector, varargin_fdr{:}, 'M', m, 'P0', randomP_fd_skeleton_vector, 'M0', m0);
else
    [fdr_skeleton_vector q_skeleton_vector pi0] = org.mensxmachina.commons.stats.fdr(p_skeleton_vector, varargin_fdr{:}, 'M', m, 'P0', randomP_fd_skeleton_vector, 'M0', m0);
end

if ~isfield(parser.Unmatched, 'Thresholds')

    % create a vector of the same size as the original output skeleton vector and
    % fill the elements corresponding to edges with the corresponding 
    % elements in the FDR vector
    fdr_vector = sparse(skeleton_vector_ind, 1, fdr_skeleton_vector, length(skeleton_vector), 1);

    if nargout > 1
        q_vector = sparse(skeleton_vector_ind, 1, q_skeleton_vector, length(skeleton_vector), 1);
    end

    % convert FDR vector to matrix
    fdr = org.mensxmachina.commons.graph.vector2skeleton(fdr_vector);

    if nargout > 1
        % convert q-values vector to matrix
        q = org.mensxmachina.commons.graph.vector2skeleton(q_vector);
    end

else

    fdr = fdr_skeleton_vector;

    if nargout > 1
        q = q_skeleton_vector;
    end
    
end

end