function v = skeletonnfp(outputSkeleton, p, targetSkeleton, t)
%SKELETONNFP Number of falsely discovered undirected edges (false positives)
%   SKELETONSTEPUP is a wrapper for ORG.MENSXMACHINA.COMMONS.STATS.STEPUP
%   specialized for Probabilistic Graphical Model (PGM) output skeleton
%   identification. Skeleton identification is viewed as a multiple testing
%   procedure, the null hypotheses being the absence of undirected edges.
%
%   V = SKELETONNFP(OUTPUTSKELETON, P, TARGETSKELETON) calculates the 
%   number V of falsely discovered undirected edges (false positives)
%   corresponding to undirected edges in output skeleton OUTPUTSKELETON
%   with p-values P, given the target skeleton TARGETSKELETON. The number 
%   of falsely discovered undirected edges corresponding to an undirected
%   edge is the one obtained when keeping only the undirected edges with
%   p-values less than or equal to the p-value of that undirected edge and
%   throwing the rest away. OUTPUTSKELETON and TARGETSKELETON are M-by-M
%   sparse adjacency matrices that represent the skeleton, i.e. the graph 
%   with undirected edges in place of directed edges, of a PGM's graph. P
%   is an M-by-M sparse matrix containing the p-values of the null
%   hypotheses of absence of the edges in OUTPUTSKELETON. P contains the
%   p-value of each edge and 0 for each non-edge in OUTPUTSKELETON. V is an
%   M-by-M sparse matrix that contains the number of falsely discovered
%   undirected edges of each edge and 0 for each non-edge in 
%   OUTPUTSKELETON.
%
%   V = SKELETONNFP(OUTPUTSKELETON, P, TARGETSKELETON, T) calculates the
%   number V of false positives corresponding to each p-value threshold in
%   T. T is a numeric real vector. V is a vector of the same size as T.
%
%   See also ORG.MENSXMACHINA.COMMONS.STATS.NFP,
%   ORG.MENSXMACHINA.PGM.SKELETONNP, ORG.MENSXMACHINA.PGM.SKELETONRLZDFPR, 
%   ORG.MENSXMACHINA.PGM.SKELETONRLZDFDR, 
%   ORG.MENSXMACHINA.PGM.SKELETONRLZDPWR  

import org.mensxmachina.commons.*;
import org.mensxmachina.commons.graph.*;
import org.mensxmachina.commons.stats.*;

assert( issparse(outputSkeleton) && issquarematrix(outputSkeleton), ...
        'org:menxmachina:pgm:skeletonnfp:BadOutputSkeleton', ...
        'OUTPUTSKELETON must an M-by-M sparse matrix.');

assert( issparse(p) && isequal(size(p), size(outputSkeleton)), ...
        'org:menxmachina:pgm:skeletonnfp:BadP', ...
        'P must a matrix of the same size as OUTPUTSKELETON.');

assert( isequal(size(targetSkeleton), size(outputSkeleton)), ...
        'org:menxmachina:pgm:skeletonnfp:TrueSkeletonSizeNotOutputSkeletonSize', ...
        'TRUESKELETON must be of the same size as OUTPUTSKELETON.');

% convert output skeleton to vector
outputSkeleton_vector = skeleton2vector(outputSkeleton);

% convert maximum p-value matrix to vector
p_vector = skeleton2vector(p);

% convert true output skeleton to vector
targetSkeleton_vector = skeleton2vector(targetSkeleton);

% get edge linear indices in output skeleton vector
outputSkeleton_vector_ind = find(outputSkeleton_vector);

% keep only maximum p-values corresponding to edges and convert to full
p_outputSkeleton_vector = full(p_vector(outputSkeleton_vector_ind));

% keep only possible edges that are edges in output skeleton and convert to full
targetSkeleton_outputSkeleton_vector = full(targetSkeleton_vector(outputSkeleton_vector_ind));

if nargin == 3
    t = p_outputSkeleton_vector;
end

% compute V corresponding to these edges
v_outputSkeleton_vector = nfp(p_outputSkeleton_vector, targetSkeleton_outputSkeleton_vector, t);

if nargin == 3

    % create output skeleton V vector
    v_vector = sparse(outputSkeleton_vector_ind, 1, v_outputSkeleton_vector, length(outputSkeleton_vector), 1);

    % convert V vector to matrix
    v = vector2skeleton(v_vector);

else % thresholds given
    
    v = v_outputSkeleton_vector;
    
end

end