function [tpr fpr thresholds] = skeletonroc(targetskeleton, p)
%SKELETONROC Skeleton learner receiver operating characteristic
%
%   SKELETONROC is a version of the ORG.MENSXMACHINA.COMMONS.STATS.ROC
%   function specialized for skeleton learning. A skeleton learning task
%   can be viewed as a binary classification task, where the possible edges
%   are classified as edges (positives) or non-edges (negatives).
%
%   [TPR,FPR,THRESHOLD] = SKELETONROC(TARGETSKELETON, P) where 
%   TARGETSKELETON is an adjacency matrix and P is the matrix of maximum
%   p-values, of the same size as TARGETSKELETON, as returned by
%   ORG.MENSXMACHINA.PGM.MMPCSKELETON targeting TARGETSKELETON and with
%   parameter 'includeNonSkeletonStats' set to true, calculates for each threshold
%   (here the thresholds are the negative maximum p-values) the true 
%   positive and false positive rate of the skeleton learner and returns
%   them in row vectors TPR and FPR, while the corresponding thresholds 
%   are returned in the THRESHOLD row vector. Each pair of values (TPR(k),
%   FPR(k)) defines a point in the Receiver operating characteristic (ROC)
%   curve.
%
%   Example:
%
%       % load ALARM network samples
%       load mxm_bnet_alarm_sample; % loads 'a' dataset
%
%       % learn skeleton
%       [skeleton p stat] = ...
%           org.mensxmachina.pgm.mmpcskeleton(a, 'includeNonSkeletonStats', true);
%
%       % load ALARM network
%       load mxm_bnet_alarm_net; % loads 'net' network
%
%       % get true skeleton
%       targetskeleton = net.skeleton;
%
%       % calculate ROC
%       [tpr fpr threshold] = ...
%           org.mensxmachina.pgm.skeletonroc(targetskeleton, p)
%
%   See also: 
%       ORG.MENSXMACHINA.COMMONS.STATS.PROC,
%       ORG.MENSXMACHINA.PGM.PLOTSKELETONROC,
%       ORG.MENSXMACHINA.PGM.SKELETONAUC 

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

% convert skeleton to vector
targets = org.mensxmachina.commons.graph.skeleton2vector(targetskeleton);

% convert maximum p-value matrix to vector
outputs = -org.mensxmachina.commons.graph.skeleton2vector(p);

[tpr fpr thresholds] = ...
    org.mensxmachina.commons.stats.roc(targets, outputs);

end