function varargout = skeletonstatstats(skeleton, stat, varargin)
%SKELETONSTATSTATS Skeleton undirected edge statistic statistics
%   MEAN = SKELETONSTATSTATS(SKELETON, STAT) returns the mean of 
%   statistics STAT (e.g. number of tests considered/attempted/succeeded)
%   of undirected edges in SKELETON. SKELETON is a M-by-M sparse adjacency
%   matrix that represents the skeleton, i.e. the graph with undirected 
%   edges in place of directed edges, of a PGM's graph. STAT is an M-by-M
%   sparse matrix which contains the statistic of each edge and 0 for each
%   non-edge in SKELETON.
%
%   [A,B,...] = SKELETONSTATSTATS(SKELETON, STAT, WHICHSTATS)
%   returns the statistics specified in WHICHSTATS. See GRPSTATS for
%   acceptable values of WHICHSTATS.

import org.mensxmachina.commons.*;
import org.mensxmachina.commons.graph.*;
import org.mensxmachina.commons.stats.*;

assert( issparse(skeleton) && issquarematrix(skeleton), ...
        'org:menxmachina:pgm:skeletonstatstats:BadSkeleton', ...
        'SKELETON must an M-by-M sparse matrix.');

assert( issparse(stat) && isnumeric(stat), ...
        'org:menxmachina:pgm:skeletonstatstats:NonNumericStat', ...
        'STAT must be sparse and numeric.');
    
assert( isequal(size(stat), size(skeleton)), ...
        'org:menxmachina:pgm:skeletonstatstats:StatSizeNotSkeletonSize', ...
        'STAT must a matrix of the same size as SKELETON.');

% convert skeleton to vector
skeleton_vector = skeleton2vector(skeleton);

% convert realized measure matrix to vector
rlzdMsr_vector = skeleton2vector(stat);

% get edge linear indices in skeleton vector
skeleton_vector_ind = find(skeleton_vector);

% keep only realized measures corresponding to edges and convert to full
x = full(rlzdMsr_vector(skeleton_vector_ind));

% compute stats
if nargout == 0
    if nargin < 3
        grpstats(x, group);
    else
        grpstats(x, group, whichstats); 
    end
else
    if nargin < 3
        eval(sprintf('[%s] = grpstats(x);', num2str(1:nargout, 'varargout{%d} ')));
    else
        eval(sprintf('[%s] = grpstats(x, [], varargin{:});', num2str(1:nargout, 'varargout{%d} ')));
    end
end

end