function [row col] = skeletonstepup(outputSkeleton, p, err, threshold)
%SKELETONSTEPUP Skeleton identification p-value step-up procedure
%   SKELETONSTEPUP is a wrapper for ORG.MENSXMACHINA.COMMONS.STATS.STEPUP
%   specialized for Probabilistic Graphical Model (PGM) output skeleton
%   identification. Skeleton identification is viewed as a multiple testing
%   procedure, the null hypotheses being the absence of undirected edges.
%
%   [ROW COL] = SKELETONSTEPUP(OUTPUSKELETON, P, ERR, THRESHOLD) returns
%   the cutoff undirected edge lower triangular row and column indices ROW
%   and COL corresponding to thresholds THRESHOLD on error ERR 
%   corresponding to undirected edges in output skeleton OUTPUSKELETON
%   with p-values P. The cutoff indices corresponding to a threshold are
%   the indices of the undirected edge whose corresponding error is less
%   than or equal to the threshold. OUTPUTSKELETON is an M-by-M sparse
%   adjacency matrix that represents the skeleton, i.e. the graph with
%   undirected edges in place of directed edges, of a PGM's graph. P and
%   ERR are M-by-M sparse matrices containing the p-values of the null
%   hypotheses of absence and errors respectively of the edges in
%   OUTPUTSKELETON. P and ERR contain the p-value and error respectively
%   of each edge and 0 for each non-edge in OUTPUTSKELETON. THRESHOLD is a
%   numeric real vector. ROW and COL are column vectors of the same length
%   as THRESHOLD. ROW and COL are NaN where THRESHOLD is NaN.
%
%   See also ORG.MENSXMACHINA.COMMONS.STATS.STEPUP, 
%   ORG.MENSXMACHINA.COMMONS.PGM.SKELETONSTEPUPSTATS

import org.mensxmachina.commons.stats.*;
import org.mensxmachina.commons.graph.*;

assert( issparse(outputSkeleton) && ndims(outputSkeleton) == 2 && ...
        size(outputSkeleton, 1) == size(outputSkeleton, 2), ...
        'org:menxmachina:pgm:skeletonfprthresholderr:BadOutputSkeleton', ...
        'OUTPUTSKELETON must an M-by-M sparse matrix.');

assert( issparse(p) && isnumeric(p) && isequal(size(p), size(outputSkeleton)), ...
        'org:menxmachina:pgm:skeletonfprthresholderr:BadP', ...
        'P must a sparse matrix of the same size as OUTPUTSKELETON.');

assert( issparse(err) && isnumeric(err) && isequal(size(err), size(outputSkeleton)), ...
        'org:menxmachina:pgm:skeletonfprthresholderr:BadErr', ...
        'ERR must be a sparse matrix of the same size as OUTPUTSKELETON.');

% Note: THRESHOLD will be validated in MULTTESTERRTHRESHOLDPCUTOFFIND

% convert output skeleton matrix to vector
outputSkeleton_vector = org.mensxmachina.commons.graph.skeleton2vector(outputSkeleton);

% convert maximal p-value matrix to vector
p_vector = org.mensxmachina.commons.graph.skeleton2vector(p);

% convert error matrix to vector
err_vector = org.mensxmachina.commons.graph.skeleton2vector(err);

% get edge linear indices in output skeleton vector
outputSkeleton_vector_ind = find(outputSkeleton_vector);

% keep only maximal p-values corresponding to edges and convert to full
p_outputSkeleton_vector = full(p_vector(outputSkeleton_vector_ind));

% keep only error corresponding to edges and convert to full
err_outputSkeleton_vector = full(err_vector(outputSkeleton_vector_ind));

% get cutoff index among these edges
ind = stepup(p_outputSkeleton_vector, err_outputSkeleton_vector, threshold);

% get cutoff index among all possible edges
ind = outputSkeleton_vector_ind(ind);

% convert to lower triangular row and column indices
[row col] = vector2skeletonind(length(outputSkeleton_vector), ind);

end