function varargout = skeletonstepupstats(outputSkeleton, p, rlzdMsr, err, threshold, varargin)
%SKELETONSTEPUPSTATS Skeleton identification measure summary statistics at each error threshold
%   SKELETONSTEPUP is a wrapper for
%   ORG.MENSXMACHINA.COMMONS.STATS.STEPUPSTATS specialized for
%   Probabilistic Graphical Model (PGM) output skeleton identification.
%   Skeleton identification is viewed as a multiple testing procedure, the
%   null hypotheses being the absence of undirected edges.
%
%   MEANS = SKELETONSTEPUPSTATS(OUTPUSKELETON, P, RLZDMSR, ERR, THRESHOLD)
%   returns the mean of several realized measures (e.g. realized False
%   Positive Rate FPR, realized power) among applications, on several
%   output skeletons, of multiple testing step-up procedures each
%   controlling an error rate (e.g. the False Discovery Rate FDR). 
%   OUTPUSKELETON is a 1-by-N or N-by-1 cell array of sparse adjacency
%   matrices each representing a skeleton, i.e. the graph with undirected
%   edges in place of directed edges, of a PGM's graph. P and ERR are
%   1-by-N or N-by-1 cell arrays of sparse matrices containing the p-values
%   of the null hypotheses of absence and errors respectively of the
%   undirected edges in the corresponding skeletons in OUTPUTSKELETON.
%   RLZDMSR is a N-by-M or M-by-N cell array of sparse matrices containing
%   the values of M realized measures for the undirected edges in the
%   corresponding skeletons in OUTPUTSKELETON. Corresponding matrices of
%   OUTPUTSKELETON, P, RLZDMSR and ERR are of the same size. THRESHOLD is a
%   numeric vector of length L with real and / or NaN elements. MEANS is a
%   L-by-M matrix with NaN rows where THRESHOLD is NaN. The means
%   corresponding to a threshold are the means, along skeletons, of error
%   rates at the undirected edge with the maximal p-value among the
%   undirected edges of each skeleton with corresponding error rate less
%   than or equal to the threshold. If no such undirected edge exists for a
%   skeleton and a threshold, 0 is used as the default realized value of
%   the realized measure.
%
%   When M==N, rows of RLZDMSR correspond to skeletons and columns to
%   measures.
%
%   [A,B,...] = SKELETONSTEPUPSTATS(OUTPUSKELETON, P, RLZDMSR, ERR, THRESHOLD, WHICHSTATS)
%   returns the statistics specified in WHICHSTATS. See GRPSTATS for
%   acceptable values of WHICHSTATS. The size of each output A, B, ... is
%   L-by-M-by-nvals, where nvals is the number of values returned by the
%   function for values for a single threshold in one measure of RLZDMSR. 
%   If N==1, then the size of each output A, B, .... is M-by-nvals.
%
%   [A,B,...] = SKELETONSTEPUPSTATS(OUTPUSKELETON, P, RLZDMSR, ERR, THRESHOLD, WHICHSTATS, RLZDMSRDEFAULT)
%   where RLZDMSRDEFAULT is a numeric real vector of length M uses the
%   values in RLZDMSRDEFAULT as the default realized measure values. If
%   RLZDMSRDEFAULT is a scalar, it is replicated into a vector.
%
%   See also ORG.MENSXMACHINA.COMMONS.STATS.STEPUPSTATS, 
%   ORG.MENSXMACHINA.PGM.SKELETONSTEPUPSTATS


%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

import org.mensxmachina.commons.*;
import org.mensxmachina.commons.graph.*;
import org.mensxmachina.commons.stats.*;

% start input parsing

ip = inputParser; % Create an instance of the inputParser class
ip.FunctionName = 'ORG.MENSXMACHINA.PGM.SKELETONSTEPUPSTATS';

ip.addRequired('outputSkeleton', @(outputSkeleton) iscell(outputSkeleton) && all(cellfun(@(iOutputSkeleton) issparse(iOutputSkeleton) && issquarematrix(iOutputSkeleton), outputSkeleton)));
ip.addRequired('p', @(outputSkeleton) iscell(p) && all(cellfun(@(iP) issparse(iP) && isnumeric(iP) && issquarematrix(iP), p)));
ip.addRequired('rlzdMsr', @(rlzdMsr) iscell(rlzdMsr) && all(all(cellfun(@(iMeasure) issparse(iMeasure) && isnumeric(iMeasure) && issquarematrix(iMeasure), rlzdMsr))));
ip.addRequired('err', @(outputSkeleton) iscell(err) && all(cellfun(@(iErr) issparse(iErr) && isnumeric(iErr) && issquarematrix(iErr), err)));

ip.parse(outputSkeleton, p, rlzdMsr, err);
    
% Note: ERRTHRESHOLD and WHICHSTATS will be validated by MULTTESTALPHARLZDMSRSTATS

assert(length(p) == length(outputSkeleton), ...
   'org:mensxmachina:pgm:skeletonfprthresholdrlzdmsrstats:PLengthNotSkeletonLength', ...
   'P must be of the same length as OUTPUSKELETON.'); 

assert(size(rlzdMsr, 1) == length(p) || size(rlzdMsr, 2) == length(p), ...
   'org:mensxmachina:pgm:skeletonfprthresholdrlzdmsrstats:PMeasureDimensionMismatch', ...
   'The dimensions of P and MEASURE do not match.');

if size(rlzdMsr, 1) ~= size(rlzdMsr, 2) && size(rlzdMsr, 2) == length(p)
    rlzdMsr = rlzdMsr'; % transpose so that measures correspond to columns
end

assert( all(arrayfun(@(i) all(cellfun(@(ijMeasure) isequal(size(ijMeasure), size(p{i})), rlzdMsr(i, :))), 1:length(p))), ...
   'org:mensxmachina:pgm:skeletonfprthresholdrlzdmsrstats:PMeasureElementsDimensionMismatch', ...
   'The dimensions of the elements of P and MEASURE do not match.'); 

% convert skeleton matrix to vector
skeleton_vector = cellfun(@skeleton2vector, outputSkeleton, 'UniformOutput', false);

% convert maximum p-value matrix to vector
p_vector = cellfun(@skeleton2vector, p, 'UniformOutput', false);

% convert each measure matrix to vector
rlzdMsr_vector = cellfun(@skeleton2vector, rlzdMsr, 'UniformOutput', false);

% convert error matrix to vector
err_vector = cellfun(@skeleton2vector, err, 'UniformOutput', false);

for i=1:length(skeleton_vector) % for each skeleton

    % get edge linear indices in skeleton vector
    skeleton_vector_ind = find(skeleton_vector{i});

    % keep only maximum p-values corresponding to edges and convert to full
    p_vector{i} = full(p_vector{i}(skeleton_vector_ind));

    for j=1:size(rlzdMsr_vector, 2) % for each measure

        % keep only measure values corresponding to edges and convert to full
        rlzdMsr_vector{i, j} = full(rlzdMsr_vector{i, j}(skeleton_vector_ind));

    end

    % keep only errors corresponding to edges and convert to full
    err_vector{i} = full(err_vector{i}(skeleton_vector_ind));

end

% end input parsing

if nargout == 0
    stepupstats(p_vector, rlzdMsr_vector, err_vector, threshold, varargin{:});
else
    eval(sprintf('[%s] = stepupstats(p_vector, rlzdMsr_vector, err_vector, threshold, varargin{:});', num2str(1:nargout, 'varargout{%d} ')));
end

end