function bnet = xdslread(filename, ClassNames)
%XDSLREAD Read Categorical Bayesian Network from XDSL file
%
%   BNET = XDSLREAD(FILENAME) reads an XDSL file FILENAME, returning
%	the data in the file as an object of the CATEGORICALBAYESIANNETWORK 
%   class with all nodes nominal. XDSL is an Bayesian Network XML format
%   used by the GeNIe & SMILE software package
%   (http://genie.sis.pitt.edu/).
%
%   XDSLREAD uses the CPT IDs in the XDSL file as the variable names and
%   the state IDs as the level labels.
%
%   BNET = XDSLREAD(FILENAME, CLASSNAMES) reads an XDSL file FILENAME,
%   returning the data in the file as a CATEGORICALBAYESIANNETWORK object 
%   whose node classes are defined in CLASSNAMES. CLASSNAMES can be 
%   'nominal' or 'ordinal', resulting in all nodes to be of that class, or
%   a cell array of 'nominal' or 'ordinal' defining the class of each node
%   separately.
%
%   Example:
%
%       alarm_bnet = org.mensxmachina.bifread('barley.bif');
%
%   See also
%       ORG.MENSXMACHINA.PGM.CATEGORICALBAYESIANNETWORK,
%       ORG.MENSXMACHINA.PGM.DSL2PGM, ORG.MENSXMACHINA.PGM.XDSLREAD

% References: 
% [1] http://genie.sis.pitt.edu/wiki/Appendices:_XDSL_File_Format_-_XML_Schema_Definitions

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

% open BIF file
xDoc = xmlread(filename);

% get smile nodes
smileNodes = xDoc.getElementsByTagName('smile');

if smileNodes.getLength > 0 % there are smile nodes

    firstSmileNode = smileNodes.item(0); % get the first one
    
    % get cpt nodes in the smile node
    cptNodes = firstSmileNode.getElementsByTagName('cpt');
    numCptNodes = cptNodes.getLength;
    
    cpt = struct( ...
        'id', {cell(1, numCptNodes)}, ...
        'state', {cell(1, numCptNodes)}, ...
        'parents', {cell(1, numCptNodes)}, ...
        'probabilities', {cell(1, numCptNodes)} ...
        );

    for i=1:numCptNodes % for each cpt node in the smile node

        cptNode = cptNodes.item(i-1);

        cpt.id{i} = char(cptNode.getAttribute('id'));

        stateNodes = cptNode.getElementsByTagName('state');
        numStateNodes = stateNodes.getLength;

        cpt.state{i} = struct('id', {cell(1, numStateNodes)}, 'label', {cell(1, numStateNodes)});
        cpt.state{i} = struct('id', {cell(1, numStateNodes)});

        for j=1:numStateNodes % for each state node in the cpt node

            stateNode = stateNodes.item(j-1);

            % get state id
            cpt.state{i}.id{j} = char(stateNode.getAttribute('id'));

            if stateNode.hasAttribute('label')
                cpt.state{i}.label{j} = char(stateNode.getAttribute('label'));
            else
                % set state id as the label too
                cpt.state{i}.label{j} = cpt.state{i}.id{j};
            end

        end

        parentsNodes = cptNode.getElementsByTagName('parents');
        numParentsNodes = parentsNodes.getLength;

        if numParentsNodes > 0
            lastParentsNode = parentsNodes.item(numParentsNodes-1);
            cpt.parents{i} = regexp(char(lastParentsNode.getFirstChild.getData), ' ', 'split');
        else % root node
            cpt.parents{i} = {};
        end
        
        probabilitiesNodes = cptNode.getElementsByTagName('probabilities');
        probabilitiesNode = probabilitiesNodes.item(0);

        cpt.probabilities{i} = str2num(char(probabilitiesNode.getFirstChild.getData));

    end
    
    % get node nodes in the smile node 
    % (actually, inside the extensions node)
    nodeNodes = firstSmileNode.getElementsByTagName('node');
    numNodeNodes = nodeNodes.getLength;
    
    node = struct( ...
        'id', {cell(1, numCptNodes)}, ...
        'name', {cell(1, numCptNodes)} ...
        );

    for i=1:numNodeNodes % for each node node in the smile node

        nodeNode = nodeNodes.item(i-1);

        node.id{i} = char(nodeNode.getAttribute('id'));
        
        nameNodes = nodeNode.getElementsByTagName('name');
        nameNode = nameNodes.item(0);
        
        node.name{i} = char(nameNode.getFirstChild.getData);
        
    end
    
    % create the constructor input arguments
    
    numLevels = zeros(1, numCptNodes);
    
    parents = cell(1, numCptNodes);
    edgeFrom = [];
    edgeTo = [];
    
    varNames = cell(1, numCptNodes);
    
    Param = cell(1, numCptNodes);
    
    labels = cell(1, numCptNodes);
    
    for i=1:numCptNodes % for each cpt
        
        % assign the number of states to the number of levels
        numLevels(i) = length(cpt.state{i}.id);
        
        % assign state labels to the level labels
        labels{i} = cpt.state{i}.label;
        
        % assign state labels to the level IDs
        %labels{i} = cpt.state{i}.id;
        
        % store the parent global indices
        parents{i} = arrayfun(@(parent) find(strcmp(parent, cpt.id), 1), cpt.parents{i});
            
        % store the edges
        edgeFrom = [edgeFrom parents{i}];
        edgeTo = [edgeTo repmat(i, 1, length(parents{i}))];
        
        % find index of cpt in node
        nodeIndInNode = find( strcmp(cpt.id{i}, node.id), 1);
        
        if ~isempty(nodeIndInNode) % there is a node with the same id
            % assign node name to the variable name
            varNames{i} = genvarname(node.name{nodeIndInNode});
        else
            % assign cpt id to the variable name
            varNames{i} = cpt.id{i};
        end
        
    end
    
    if length(unique(varNames)) < length(varNames) % there are duplicate node names
        
        % assign cpt ids to the variable names
        varNames = cpt.id;
        
    end
    
    for i=1:numCptNodes % for each cpt
        
        if isempty(parents{i})
            parentNumLevels = 1;
        else
            parentNumLevels = numLevels(parents{i});
        end
        
        % fix cpt probabilities so they sum to 1
        for j=numLevels(i):numLevels(i):length(cpt.probabilities{i})
            cpt.probabilities{i}(j) = cpt.probabilities{i}(j) - sum(cpt.probabilities{i}((j-numLevels(i)+1):j)) + 1;
        end
        
        % assign cpt probabilities to the BN node parameters
        Param{i} = reshape(cpt.probabilities{i}, [numLevels(i) parentNumLevels(end:-1:1)]);
        
        if ~isempty(parents{i})
            
            % reverse parent dimensions
            Param{i} = permute(Param{i}, [1 (length(parents{i}) + 1):-1:2 ]);
        
            % sort parent dimensions according to the parents global order
            [~, parents_global_order] = sort(parents{i});
            Param{i} = permute(Param{i}, [1 (parents_global_order + 1)]);
            
        end
        
    end
        
    % create the graph
    G = sparse(edgeFrom, edgeTo, 1, numCptNodes, numCptNodes);
    
    if nargin==1
        ClassNames = 'nominal';
    end
    
    % create the network object 
    bnet = org.mensxmachina.pgm.CategoricalBayesianNetwork( ...
        G, numLevels, Param, 'VarNames', varNames, 'Labels', labels, 'ClassNames', ClassNames);
    
else
    
    bnet = [];
    
end

end