function bnet = xmlbifread(filename, ClassNames)
%XMLBIFREAD Read Categorical Bayesian Network from XMLBIF 0.3 file
%
%   BNET = XMLBIFREAD(FILENAME) reads an XDSL file FILENAME, returning
%	the data in the file as an object of the CATEGORICALBAYESIANNETWORK 
%   class with all nodes nominal.
%
%   XMLBIFREAD uses the CPT IDs in the XDSL file as the variable names and
%   the outcome IDs as the level labels.
%
%   BNET = XMLBIFREAD(FILENAME, CLASSNAMES) reads an XDSL file FILENAME,
%   returning the data in the file as a CATEGORICALBAYESIANNETWORK object 
%   whose node classes are defined in CLASSNAMES. CLASSNAMES can be 
%   'nominal' or 'ordinal', resulting in all nodes to be of that class, or
%   a cell array of 'nominal' or 'ordinal' defining the class of each node
%   separately.
%
%   Example:
%
%       alarm_bnet = org.mensxmachina.xmlbifread('alarm.xml');
%
%   See also
%       ORG.MENSXMACHINA.PGM.CATEGORICALBAYESIANNETWORK,
%       ORG.MENSXMACHINA.PGM.DSL2PGM, ORG.MENSXMACHINA.PGM.XDSLREAD

% References: 
% [1] http://www.poli.usp.br/p/fabio.cozman/Research/InterchangeFormat/index.html

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

% open XMLBIF file
xDoc = xmlread(filename);

% get NETWORK nodes
networkNodes = xDoc.getElementsByTagName('NETWORK');

if networkNodes.getLength > 0 % there are NETWORK nodes

    firstNetworkNode = networkNodes.item(0); % get the first one
    
    % get NAME
    nameNodes = firstNetworkNode.getElementsByTagName('NAME');
    nameNode = nameNodes.item(0);
    name = char(nameNode.getFirstChild.getData);
    
    % get VARIABLE nodes
    variableNodes = firstNetworkNode.getElementsByTagName('VARIABLE');
    numVariableNodes = variableNodes.getLength;
    
    variable = struct( ...
        'name', {cell(1, numVariableNodes)}, ...
        'outcome', {cell(1, numVariableNodes)} ...
        );

    for i=1:numVariableNodes % for each VARIABLE node in the NETWORK node

        % (ignore TYPE attribute and PROPERTY elements)
        
        variableNode = variableNodes.item(i-1);

        % get NAME
        nameNodes = variableNode.getElementsByTagName('NAME');
        nameNode = nameNodes.item(0);
        variable.name{i} = char(nameNode.getFirstChild.getData);

        outcomeNodes = variableNode.getElementsByTagName('OUTCOME');
        numOutcomeNodes = outcomeNodes.getLength;

        variable.outcome{i} = cell(1, numOutcomeNodes);

        for j=1:numOutcomeNodes % for each OUTCOME node in the VARIABLE node

            outcomeNode = outcomeNodes.item(j-1);
            
            % get OUTCOME
            variable.outcome{i}{j} = char(outcomeNode.getFirstChild.getData);

        end

    end
    
    % get DEFINITION nodes
    definitionNodes = firstNetworkNode.getElementsByTagName('DEFINITION');
    numDefinitionNodes = definitionNodes.getLength;
    
    definition = struct( ...
        'for', {cell(1, numDefinitionNodes)}, ...
        'given', {cell(1, numDefinitionNodes)}, ...
        'table', {cell(1, numDefinitionNodes)} ...
        );

    for i=1:numDefinitionNodes % for each DEFINITION node

        definitionNode = definitionNodes.item(i-1);

        % get FOR
        forNodes = definitionNode.getElementsByTagName('FOR');
        forNode = forNodes.item(0);
        definition.for{i} = char(forNode.getFirstChild.getData);

        % get GIVEN nodes
        givenNodes = definitionNode.getElementsByTagName('GIVEN');
        numGivenNodes = givenNodes.getLength;
        
        definition.given{i} = cell(1, numGivenNodes);
        
        for j=1:numGivenNodes % for each GIVEN node
        
            givenNode = givenNodes.item(j-1);
            
            % get GIVEN
            definition.given{i}{j} = char(givenNode.getFirstChild.getData);
            
        end

        % get TABLE
        tableNodes = definitionNode.getElementsByTagName('TABLE');
        tableNode = tableNodes.item(0);
        definition.table{i} = char(tableNode.getFirstChild.getData);
        
    end
    
    % create the constructor input arguments
    
    numLevels = zeros(1, numVariableNodes);
    
    parents = cell(1, numVariableNodes);
    edgeFrom = [];
    edgeTo = [];
    
    Param = cell(1, numVariableNodes);
    
    labels = cell(1, numVariableNodes);
    
    for i=1:numVariableNodes % for each variable
        
        % assign the number of outcomes to the number of levels
        numLevels(i) = length(variable.outcome{i});
        
        % assign outcomes to the level labels
        labels{i} = variable.outcome{i};
        
        % store the parent global indices
        parents{i} = cellfun(@(ijFor) find(strcmp(ijFor, variable.name), 1), definition.given{i});
            
        % store the edges
        edgeFrom = [edgeFrom parents{i}];
        edgeTo = [edgeTo repmat(i, 1, length(parents{i}))];
        
    end
    
    for i=1:numVariableNodes % for each variable
        
        if isempty(parents{i})
            parentNumLevels = 1;
        else
            parentNumLevels = numLevels(parents{i});
        end
        
        % assign definition table to the BN node parameters
        table = str2num(definition.table{i});
        % fix probabilities so they sum to 1
        for j=numLevels(i):numLevels(i):length(table)
            table(j) = table(j) - sum(table((j-numLevels(i)+1):j)) + 1;
        end
        Param{i} = reshape(table, [numLevels(i) parentNumLevels(end:-1:1)]);
        
        if ~isempty(parents{i})
            
            % reverse parent dimensions
            Param{i} = permute(Param{i}, [1 (length(parents{i}) + 1):-1:2 ]);
        
            % sort parent dimensions according to the parents global order
            [~, parents_global_order] = sort(parents{i});
            Param{i} = permute(Param{i}, [1 (parents_global_order + 1)]);
            
        end
        
    end
        
    % create the graph
    G = sparse(edgeFrom, edgeTo, 1, numVariableNodes, numVariableNodes);
    
    if nargin==1
        ClassNames = 'nominal';
    end
    
    % create the network object 
    bnet = org.mensxmachina.pgm.CategoricalBayesianNetwork( ...
        G, numLevels, Param, 'VarNames', variable.name, 'Labels', labels, 'ClassNames', ClassNames, 'Description', name);
    
else
    
    bnet = [];
    
end

end